function [StIt,It1_node1,It1_node2,It2_node2,It2_node1] = SIR2node(N1, N2, I01, I02, R01, R02, t0, tf, dt, b1, gam1, b2, gam2, alpha12, alpha21)
% N1, N2 -- number of initial trees in nodes 1 and 2
% I01, I02 -- number of initial infected
% R01, R02 -- number of initial removed
% t0 -- start time
% tf -- end time
% dt -- time increment
% b1, b2 -- infectivity rate
% gam1, gam2 -- removal rate
% a12 -- movement 'push' from node 1 to node 2
% a21 -- movement 'push' from node 2 to node 1

S01=N1-I01-R01; %susceptible initial
S02=N2-I02-R02; %susceptible initial
t=t0:dt:tf; %sets time interval
n=max(size(t)); %determines number of iterations
sqrtdt=sqrt(dt);
St1=zeros(n,1); St2=zeros(n,1); %initializes vector and sets the initial condition
It1=zeros(n,1); It2=zeros(n,1); %initializes vector and sets the initial condition
St1(1)=S01; St2(1)=S02;
It1(1)=I01; It2(1)=I02;

for i=1:n-1
W1=randn*sqrtdt;
W2=randn*sqrtdt;
W3=randn*sqrtdt;
W4=randn*sqrtdt;

St1(i+1)=St1(i)-b1*(It1(i)+alpha21*It2(i))*St1(i)*dt-sqrt(b1*(It1(i)+alpha21*It2(i))*St1(i))*W1;
St2(i+1)=St2(i)-b2*(It2(i)+alpha12*It1(i))*St2(i)*dt-sqrt(b2*(It2(i)+alpha12*It1(i))*St2(i))*W3;
It1(i+1)=It1(i)+b1*(It1(i)+alpha21*It2(i))*St1(i)*dt-gam1*It1(i)*dt+sqrt(b1*(It1(i)+alpha21*It2(i))*St1(i))*W1-sqrt(gam1*It1(i))*W2;
It2(i+1)=It2(i)+b2*(It2(i)+alpha12*It1(i))*St2(i)*dt-gam2*It2(i)*dt+sqrt(b2*(It2(i)+alpha12*It1(i))*St2(i))*W3-sqrt(gam2*It2(i))*W4;
It1_node1(i+1)=b1*It1(i)*St1(i)*dt;
It1_node2(i+1)=b1*alpha21*It2(i)*St1(i)*dt;
It2_node2(i+1)=b2*It2(i)*St2(i)*dt;
It2_node1(i+1)=b2*alpha12*It1(i)*St2(i)*dt;

    if St1(i+1)<0
    St1(i+1)=0.01;
    end 
    if St2(i+1)<0
    St2(i+1)=0.01;
    end 
    if It1(i+1)<0.01
    It1(i+1)=0.01;
    end 
    if It2(i+1)<0.01
    It2(i+1)=0.01;
    end 

end

StIt=[St1 It1 St2 It2];

end