function [xsamp,vhat,ll] = ffilter_2node(ODEdt,inter,param,a,C,F,x)
%forward filter backward sampler for inference

%inputs:
%ODEdt = time step for ODE in LNA
%inter = inter observation time (from data set)
%param = ODE parameters, theta in MCMC
%a = mean of prior over initial latent state (for tstep)
%C = variance of prior over initial latent state (for tstep)
%F = observational matrix
%x = data

%outputs:
%xsamp = within-sample posteriors 
%vhat = posterior variance
%set up and storage
d=length(a);
ODEend=inter/ODEdt+1;
n=length(x);
mt=zeros(1,d);
Vt=zeros(d);
xsamp=zeros(d,n);
mtA=zeros([d,1,n]);
VtA=zeros([d,d,n]);
atA=zeros([d,1,n]);
CtA=zeros([d,d,n]);
PtA=zeros([d,d,n]);
vhat=zeros([d,d,n]);

ll=0; %set loglikelihood to 0 initially
atA(:,:,1)=a;
CtA(:,:,1)=C;
sige=param(5); %noise parameter

for i=1:n-1

    %time step forward the ODE using the LNA
    [mt, Vt, Pt]=tstep2node(inter,ODEdt,a,C,param(1:4));
    mt=mt(:,ODEend); Vt=Vt(:,:,ODEend); Pt=Pt(:,:,ODEend);
     
    %checks on Vt:
    if abs(det(Vt(1:2,1:2)))>1e30
        i=n;
        ll=-1e40;
        break %reject anything weird
    end
    
    mtA(:,:,i+1)=mt; VtA(:,:,i+1)=Vt; PtA(:,:,i+1)=Pt;
         
    %update marginal likelihood
    ll=ll+logmvnpdf(x(i+1,:),(F'*mt)',F'*Vt*F+F'*diag(mt*sige.^2)*F);

    %update posterior mean
    a=mt+Vt*F*linsolve(F'*Vt*F+F'*diag(mt*sige.^2)*F,eye(size(F,2)))*(x(i+1,:)'-F'*mt);
    
    %some fix for small values of S and I
    a(a<0.001)=0.001;
    
    %update posterior variance
    C=Vt-Vt*F*linsolve(F'*Vt*F+F'*diag(mt*sige.^2)*F,eye(size(F,2)))*F'*Vt;
    
    atA(:,:,i+1)=a;
    CtA(:,:,i+1)=C;
end

%backward sweep
%sample end 
%only execute if forward filter ran OK
if ll~=-1e40
xsamp(:,n)=rmvn(a,C);
vhat(:,:,n)=C;

    %backward sampler
    for i=n-1:-1:1
    %calculate ahat and Chat
    ahat=atA(:,:,i)+CtA(:,:,i)*(PtA(:,:,i+1))'*linsolve(VtA(:,:,i+1),eye(4))*(xsamp(:,i+1)-mtA(:,:,i+1));
    Chat=CtA(:,:,i)-CtA(:,:,i)*(PtA(:,:,i+1))'*linsolve(VtA(:,:,i+1),eye(4))*PtA(:,:,i+1)*CtA(:,:,i);
    
    %save within sample posteriors and posterior variance
    xsamp(:,i)=rmvn(ahat,Chat);
    vhat(:,:,i)=Chat;
    end
end

end 
